@@ -9,10 +9,12 @@ import tempfile
 import warnings
 import copyreg
 from contextlib import closing, contextmanager
+
 try:
    import _pickle as pickle
 except:
    import pickle
+
 import pathlib
 import inspect
 import tensorflow as tf
@@ -182,7 +184,7 @@ def check_module_version_greater_or_equal(module, req_version_tuple, error_if_ma
 
 
 def _cpu_tag(obj):
-    if type(obj).__module__ == 'torch':
+    if type(obj).__module__ == 'tensorflow':
         return 'cpu'
 
 
