@@ -99,19 +99,20 @@ def list_images(directory, ext='jpg|jpeg|jpe|tiff|tif|bmp|png|ppm|jfif'):
 def check_same_size(*images):
     result = True
     base_shape =None
-    if isinstance(images[0],numbers.Number):
+    if isinstance(images[0],numbers.Number) or isinstance(images[0],str):
         base_shape=0
     elif  isinstance(images[0],np.ndarray):
         base_shape = TensorShape(images[0].shape)
     # check same isze
     for img in images:
-        if base_shape==0 and isinstance(img, numbers.Number):
+        if base_shape==0 and (isinstance(img, numbers.Number) or isinstance(images[0],str)):
             pass
         elif   isinstance(img,np.ndarray) and base_shape== TensorShape(img.shape):
             pass
         else:
             print(img.shape)
             result = False
+
     return True
 
 
