@@ -14,7 +14,7 @@ import numpy as np
 import torch
 import torch.nn as nn
 import torch.nn.functional as F
-from torch._six import container_abcs
+from collections import abc
 from torch.nn import init
 
 from trident.layers.pytorch_activations import get_activation, Identity
@@ -35,7 +35,7 @@ _epsilon = _session.epsilon
 
 def _ntuple(n):
     def parse(x):
-        if isinstance(x, container_abcs.Iterable):
+        if isinstance(x, abc.Iterable):
             return x
         return tuple(repeat(x, n))
 
