@@ -363,11 +363,10 @@ class SoftMax(Layer):
             self.add_noise = False
             self.noise_intensity = 0.005
         if self.training:
-            x = tf.math.log_softmax(x, self.axis)
             if self.add_noise == True:
-                noise = self.noise_intensity * tf.random.normal(shape=x.get_shape(), mean=1, stddev=1,dtype=x.dtype)
+                noise = self.noise_intensity * tf.random.uniform(shape=x.get_shape(),minval=0.0,maxval=1.0,dtype=x.dtype)
                 x = x + noise
-
+            x = tf.math.log_softmax(x, self.axis)
         else:
             x = tf.math.softmax(x, self.axis)
         return x
