@@ -3,6 +3,7 @@ from __future__ import division
 from __future__ import print_function
 
 import math
+import sys
 
 import numpy as np
 import tensorflow as tf
@@ -180,21 +181,32 @@ class _ClassificationLoss(Loss):
         else:
             pass
 
-        output_exp = exp(output)
 
-        if (ndim(output) >= 1 and 'float' in str(output.dtype) and output.min() >= 0 and output.max() <= 1):
+
+        if (ndim(output) >= 1 and 'float' in str(output.dtype) and reduce_min(output) >= 0):
             self.is_logsoftmax = False
-            self.from_logits = True
+
+            if reduce_max(output) > 1:
+                output = output / reduce_max(output)
+
+            sum_output = reduce_sum(output, axis=self.axis, keepdims=True)
+            if reduce_max(abs(sum_output - ones_like(sum_output))) < 1e-3:
+                self.from_logits = True
+            else:
+                output = output / clip(sum_output, 1e-7, 1 - 1e-7)
+                self.from_logits = True
+
             if self.auto_balance and self.label_statistics is not None:
                 output = output * to_tensor(self.label_statistics.copy())
             output = clip(output, min=1e-8, max=1 - 1e-8)
 
-        elif (ndim(output) >= 1 and 'float' in str(output.dtype) and output_exp.min() >= 0 and output_exp.max() <= 1):
+        elif (ndim(output) >= 1 and 'float' in str(output.dtype) and  reduce_max(output) <=0):
             self.is_logsoftmax = True
             self.from_logits = True
+            output = clip(output,min=-7.0,max=-1e-8)
             if self.auto_balance and self.label_statistics is not None:
                 output = output + to_tensor(np.log(self.label_statistics.copy()))
-            output = clip(output, max=- 1e-8)
+
         else:
             self.is_logsoftmax = False
             self.from_logits = False
@@ -239,6 +251,12 @@ class _ClassificationLoss(Loss):
         ##dont do aggregation
         raise NotImplementedError
 
+    def _handel_abnormal(self, loss):
+        if any_abnormal_number(loss):
+            sys.stderr.write('{0} has abnormal value,trident automatically replace these abnormal value to zero.\n'.format(self.name))
+            loss = where(is_abnormal_number(loss), ones_like(loss) * 1e-7, loss)
+        return loss
+
     def forward(self, output: Tensor, target: Tensor, **kwargs) -> 'loss':
         """
 
@@ -253,9 +271,9 @@ class _ClassificationLoss(Loss):
         try:
             output, target = self.flatten_check(output, target)
             loss = self.calculate_loss(*self.preprocess(output, target, **kwargs))
+            loss = self._handel_abnormal(loss)
             loss = self._get_reduction(loss)
-            if any_abnormal_number(loss):
-                print('')
+
             return loss
         except Exception as e:
             print(e)
